"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_customerlookupresult import (
    DtoCustomerLookupResult,
    DtoCustomerLookupResultTypedDict,
)
from .dto_metermatchingresult import (
    DtoMeterMatchingResult,
    DtoMeterMatchingResultTypedDict,
)
from .dto_pricelookupresult import DtoPriceLookupResult, DtoPriceLookupResultTypedDict
from .dto_subscriptionlineitemlookupresult import (
    DtoSubscriptionLineItemLookupResult,
    DtoSubscriptionLineItemLookupResultTypedDict,
)
from .failurepoint import FailurePoint, FailurePointTypedDict
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoDebugTrackerTypedDict(TypedDict):
    customer_lookup: NotRequired[DtoCustomerLookupResultTypedDict]
    failure_point: NotRequired[FailurePointTypedDict]
    meter_matching: NotRequired[DtoMeterMatchingResultTypedDict]
    price_lookup: NotRequired[DtoPriceLookupResultTypedDict]
    subscription_line_item_lookup: NotRequired[
        DtoSubscriptionLineItemLookupResultTypedDict
    ]


class DtoDebugTracker(BaseModel):
    customer_lookup: Optional[DtoCustomerLookupResult] = None

    failure_point: Optional[FailurePoint] = None

    meter_matching: Optional[DtoMeterMatchingResult] = None

    price_lookup: Optional[DtoPriceLookupResult] = None

    subscription_line_item_lookup: Optional[DtoSubscriptionLineItemLookupResult] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "customer_lookup",
                "failure_point",
                "meter_matching",
                "price_lookup",
                "subscription_line_item_lookup",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
