"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .debugtrackerstatus import DebugTrackerStatus
from .errors_errorresponse import ErrorsErrorResponse, ErrorsErrorResponseTypedDict
from .github_com_flexprice_flexprice_internal_domain_customer_customer import (
    GithubComFlexpriceFlexpriceInternalDomainCustomerCustomer,
    GithubComFlexpriceFlexpriceInternalDomainCustomerCustomerTypedDict,
)
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCustomerLookupResultTypedDict(TypedDict):
    customer: NotRequired[
        GithubComFlexpriceFlexpriceInternalDomainCustomerCustomerTypedDict
    ]
    error: NotRequired[ErrorsErrorResponseTypedDict]
    status: NotRequired[DebugTrackerStatus]


class DtoCustomerLookupResult(BaseModel):
    customer: Optional[GithubComFlexpriceFlexpriceInternalDomainCustomerCustomer] = None

    error: Optional[ErrorsErrorResponse] = None

    status: Optional[DebugTrackerStatus] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["customer", "error", "status"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
