"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


Provider = Literal[
    "stripe",
    "razorpay",
    "paypal",
]
r"""provider is the integration provider name (e.g., \"stripe\", \"razorpay\")"""


class DtoIntegrationEntityMappingTypedDict(TypedDict):
    r"""Integration entity mapping for external provider systems"""

    id: str
    r"""id is the external entity ID from the provider"""
    provider: Provider
    r"""provider is the integration provider name (e.g., \"stripe\", \"razorpay\")"""


class DtoIntegrationEntityMapping(BaseModel):
    r"""Integration entity mapping for external provider systems"""

    id: str
    r"""id is the external entity ID from the provider"""

    provider: Provider
    r"""provider is the integration provider name (e.g., \"stripe\", \"razorpay\")"""
