"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_createentitlementrequest import (
    DtoCreateEntitlementRequest,
    DtoCreateEntitlementRequestTypedDict,
)
from flexprice.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class DtoCreateBulkEntitlementRequestTypedDict(TypedDict):
    items: List[DtoCreateEntitlementRequestTypedDict]


class DtoCreateBulkEntitlementRequest(BaseModel):
    items: List[DtoCreateEntitlementRequest]
