"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .walletstatus import WalletStatus
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


WalletFilterOrder = Literal[
    "asc",
    "desc",
]


class WalletFilterTypedDict(TypedDict):
    alert_enabled: NotRequired[bool]
    expand: NotRequired[str]
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[WalletFilterOrder]
    sort: NotRequired[str]
    status: NotRequired[WalletStatus]
    wallet_ids: NotRequired[List[str]]


class WalletFilter(BaseModel):
    alert_enabled: Optional[bool] = None

    expand: Optional[str] = None

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[WalletFilterOrder] = None

    sort: Optional[str] = None

    status: Optional[WalletStatus] = None

    wallet_ids: Optional[List[str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_enabled",
                "expand",
                "limit",
                "offset",
                "order",
                "sort",
                "status",
                "wallet_ids",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
