"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


PaymentBehavior = Literal[
    "allow_incomplete",
    "default_incomplete",
    "error_if_incomplete",
    "default_active",
]
