"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


InvoiceBilling = Literal[
    "invoice_to_parent",
    "invoice_to_self",
]
