"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel
from typing_extensions import TypedDict


class DtoCreateGroupRequestTypedDict(TypedDict):
    entity_type: str
    lookup_key: str
    name: str


class DtoCreateGroupRequest(BaseModel):
    entity_type: str

    lookup_key: str

    name: str
