"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import UnrecognizedStr
from typing import Literal, Union


FeatureType = Union[
    Literal[
        "metered",
        "boolean",
        "static",
    ],
    UnrecognizedStr,
]
