"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .taxrateentitytype import TaxRateEntityType
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateTaxAssociationRequestTypedDict(TypedDict):
    tax_rate_code: str
    auto_apply: NotRequired[bool]
    currency: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[TaxRateEntityType]
    external_customer_id: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    priority: NotRequired[int]


class DtoCreateTaxAssociationRequest(BaseModel):
    tax_rate_code: str

    auto_apply: Optional[bool] = None

    currency: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[TaxRateEntityType] = None

    external_customer_id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    priority: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "auto_apply",
                "currency",
                "entity_id",
                "entity_type",
                "external_customer_id",
                "metadata",
                "priority",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
