"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .couponcadence import CouponCadence
from .coupontype import CouponType
from .status import Status
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCouponResponseTypedDict(TypedDict):
    amount_off: NotRequired[str]
    cadence: NotRequired[CouponCadence]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    duration_in_periods: NotRequired[int]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    max_redemptions: NotRequired[int]
    metadata: NotRequired[Dict[str, str]]
    name: NotRequired[str]
    percentage_off: NotRequired[str]
    redeem_after: NotRequired[str]
    redeem_before: NotRequired[str]
    rules: NotRequired[Dict[str, Any]]
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    total_redemptions: NotRequired[int]
    type: NotRequired[CouponType]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoCouponResponse(BaseModel):
    amount_off: Optional[str] = None

    cadence: Optional[CouponCadence] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None

    duration_in_periods: Optional[int] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    max_redemptions: Optional[int] = None

    metadata: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    percentage_off: Optional[str] = None

    redeem_after: Optional[str] = None

    redeem_before: Optional[str] = None

    rules: Optional[Dict[str, Any]] = None

    status: Optional[Status] = None

    tenant_id: Optional[str] = None

    total_redemptions: Optional[int] = None

    type: Optional[CouponType] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount_off",
                "cadence",
                "created_at",
                "created_by",
                "currency",
                "duration_in_periods",
                "environment_id",
                "id",
                "max_redemptions",
                "metadata",
                "name",
                "percentage_off",
                "redeem_after",
                "redeem_before",
                "rules",
                "status",
                "tenant_id",
                "total_redemptions",
                "type",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
