"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import UnrecognizedStr
from typing import Literal, Union


DebugTrackerStatus = Union[
    Literal[
        "unprocessed",
        "not_found",
        "found",
        "error",
    ],
    UnrecognizedStr,
]
