"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .taskstatus import TaskStatus
from flexprice.types import BaseModel
from typing_extensions import TypedDict


class DtoUpdateTaskStatusRequestTypedDict(TypedDict):
    task_status: TaskStatus


class DtoUpdateTaskStatusRequest(BaseModel):
    task_status: TaskStatus
