"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .meter_aggregation import MeterAggregation, MeterAggregationTypedDict
from .meter_filter import MeterFilter, MeterFilterTypedDict
from .resetusage import ResetUsage
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateMeterRequestTypedDict(TypedDict):
    aggregation: MeterAggregationTypedDict
    event_name: str
    name: str
    reset_usage: ResetUsage
    filters: NotRequired[List[MeterFilterTypedDict]]


class DtoCreateMeterRequest(BaseModel):
    aggregation: MeterAggregation

    event_name: str

    name: str

    reset_usage: ResetUsage

    filters: Optional[List[MeterFilter]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["filters"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
